@@ -96,10 +96,6 @@ poly      p_ISet(long i, const ring r);
 poly      p_NSet(number n, const ring r);
 
 void  p_Vec2Polys(poly v, poly**p, int *len, const ring r);
-poly  p_Vec2Poly(poly v, int k, const ring r);
-
-/// julia: vector to already allocated array (len=p_MaxComp(v,r))
-void  p_Vec2Array(poly v, poly *p, int len, const ring r);
 
 /***************************************************************
  *
@@ -216,7 +212,6 @@ void      p_Content(poly p, const ring r);
 #if 1
 // currently only used by Singular/janet
 void      p_SimpleContent(poly p, int s, const ring r);
-number    p_InitContent(poly ph, const ring r);
 #endif
 
 poly      p_Cleardenom(poly p, const ring r);
@@ -802,25 +797,17 @@ p_GetTotalDegree(const unsigned long l, const ring r, const int number_of_exps)
 /// returns a copy of p (without any additional testing)
 static inline poly p_Copy_noCheck(poly p, const ring r)
 {
-  /*assume(p!=NULL);*/
-  assume(r != NULL);
-  assume(r->p_Procs != NULL);
-  assume(r->p_Procs->p_Copy != NULL);
+  assume(r != NULL); assume(r->p_Procs != NULL); assume(r->p_Procs->p_Copy != NULL);
   return r->p_Procs->p_Copy(p, r);
 }
 
 /// returns a copy of p
 static inline poly p_Copy(poly p, const ring r)
 {
-  if (p!=NULL)
-  {
-    p_Test(p,r);
-    const poly pp = p_Copy_noCheck(p, r);
-    p_Test(pp,r);
-    return pp;
-  }
-  else
-    return NULL;
+  p_Test(p,r);
+  const poly pp = p_Copy_noCheck(p, r);
+  p_Test(pp,r);
+  return pp;
 }
 
 static inline poly p_Head(poly p, const ring r)
@@ -836,6 +823,18 @@ static inline poly p_Head(poly p, const ring r)
   return np;
 }
 
+static inline poly p_LmHead(poly p, const ring r)
+{
+  p_LmCheckPolyRing1(p, r);
+  poly np;
+  omTypeAllocBin(poly, np, r->PolyBin);
+  p_SetRingOfLm(np, r);
+  memcpy(np->exp, p->exp, r->ExpL_Size*sizeof(long));
+  pNext(np) = NULL;
+  pSetCoeff0(np, NULL);
+  return np;
+}
+
 // returns a copy of p with Lm(p) from lmRing and Tail(p) from tailRing
 static inline poly p_Copy(poly p, const ring lmRing, const ring tailRing)
 {
@@ -846,8 +845,7 @@ static inline poly p_Copy(poly p, const ring lmRing, const ring tailRing)
       return p_Copy_noCheck(p, tailRing);
 #endif
     poly pres = p_Head(p, lmRing);
-    if (pNext(p)!=NULL)
-      pNext(pres) = p_Copy_noCheck(pNext(p), tailRing);
+    pNext(pres) = p_Copy_noCheck(pNext(p), tailRing);
     return pres;
   }
   else
@@ -859,7 +857,7 @@ static inline void p_Delete(poly *p, const ring r)
 {
   assume( p!= NULL );
   assume( r!= NULL );
-  if ((*p)!=NULL) r->p_Procs->p_Delete(p, r);
+  r->p_Procs->p_Delete(p, r);
 }
 
 static inline void p_Delete(poly *p,  const ring lmRing, const ring tailRing)
@@ -881,7 +879,7 @@ static inline void p_Delete(poly *p,  const ring lmRing, const ring tailRing)
 }
 
 // copys monomials of p, allocates new monomials from bin,
-// deletes monomials of p
+// deletes monomoals of p
 static inline poly p_ShallowCopyDelete(poly p, const ring r, omBin bin)
 {
   p_LmCheckPolyRing2(p, r);
@@ -919,7 +917,7 @@ static inline poly p_Mult_nn(poly p, number n, const ring r)
     return p;
   else if (n_IsZero(n, r->cf))
   {
-    p_Delete(&p, r); // NOTE: without p_Delete - memory leak!
+    r->p_Procs->p_Delete(&p, r); // NOTE: without p_Delete - memory leak!
     return NULL;
   }
   else
@@ -1054,30 +1052,36 @@ static inline poly p_Mult_q(poly p, poly q, const ring r)
 
   if (p == NULL)
   {
-    p_Delete(&q, r);
+    r->p_Procs->p_Delete(&q, r);
     return NULL;
   }
   if (q == NULL)
   {
-    p_Delete(&p, r);
+    r->p_Procs->p_Delete(&p, r);
     return NULL;
   }
 
   if (pNext(p) == NULL)
   {
-    q = r->p_Procs->p_mm_Mult(q, p, r);
-    p_LmDelete(&p, r);
+#ifdef HAVE_PLURAL
+    if (rIsPluralRing(r))
+      q = nc_mm_Mult_p(p, q, r);
+    else
+#endif /* HAVE_PLURAL */
+      q = r->p_Procs->p_Mult_mm(q, p, r);
+
+    r->p_Procs->p_Delete(&p, r);
     return q;
   }
 
   if (pNext(q) == NULL)
   {
     p = r->p_Procs->p_Mult_mm(p, q, r);
-    p_LmDelete(&q, r);
+    r->p_Procs->p_Delete(&q, r);
     return p;
   }
 #ifdef HAVE_PLURAL
-  if (rIsNCRing(r))
+  if (rIsPluralRing(r))
     return _nc_p_Mult_q(p, q, r);
   else
 #endif
@@ -1091,7 +1095,11 @@ static inline poly pp_Mult_qq(poly p, poly q, const ring r)
 
   if (pNext(p) == NULL)
   {
-    return r->p_Procs->pp_mm_Mult(q, p, r);
+#ifdef HAVE_PLURAL
+    if (rIsPluralRing(r))
+      return nc_mm_Mult_pp(p, q, r);
+#endif
+    return r->p_Procs->pp_Mult_mm(q, p, r);
   }
 
   if (pNext(q) == NULL)
@@ -1653,27 +1661,12 @@ BOOLEAN p_ComparePolys(poly p1,poly p2, const ring r);
 static inline int p_Cmp(poly p1, poly p2, ring r)
 {
   if (p2==NULL)
-  {
-    if (p1==NULL) return 0;
     return 1;
-  }
   if (p1==NULL)
     return -1;
   return p_LmCmp(p1,p2,r);
 }
 
-static inline int p_CmpPolys(poly p1, poly p2, ring r)
-{
-  if (p2==NULL)
-  {
-    if (p1==NULL) return 0;
-    return 1;
-  }
-  if (p1==NULL)
-    return -1;
-  return p_ComparePolys(p1,p2,r);
-}
-
 
 /***************************************************************
  *
